function AM = GraphicalLasso(fileName)
% ***************************************************************
% Name:      GraphicalLasso.m
% Purpose:   This function takes a comma-separated species abundance file 
%            with frequency of taxa for each sample and estimates a sparse
%            inverse of a covariance matrix called precision matrix. If the 
%            ijth component of the precision matrix is zero then i and j
%            are conditionally independent. For reference, please see the
%            following paper:
%               O. Banerjee, L. E. Ghaoui, A. d?Aspremont. Model selection 
%               through sparse maximum likelihood estimation for multivariate 
%               gaussian or binary data. Journal of Machine Language Research 
%               101, 2007
%            Please note that the Graphical Lasso model can only be used
%            when the data is Gaussian distributed. It is this reason why
%            we take the log transform of the species abundance data 
%            before using the algorithm. If your data is already
%            gaussian-distributed, set logTransformData=0 in the code given
%            below. The algorithm uses as input the empirical
%            covariance matrix "sigma" and estimates the covariance using a
%            Block-Coordinate-Descent algorithm which is interpretted as a 
%            recursive L1-norm penalized regression.  
%            Please make sure the format of the species abundance file is
%            as follows:
%               OTUs,Sample_1,Sample2,Sample_3,Sample_4,...,Sample_n
%               OTU_1,
%               OTU_2,
%               ...
%               OTU_s
%            You can run the algorithm as
%                   AM=GraphicalLasso('filename.csv');
%            This will generate a filename.dot in the current directory
%            which you can then open in GraphViz to view the network, AM
%            will be the estimated precision matrix.
% Version:   0.1
% Authors:   Umer Zeeshan Ijaz (Umer.Ijaz@glasgow.ac.uk)
%                 http://userweb.eng.gla.ac.uk/umer.ijaz
% Created:   2012-10-23
% License:   Copyright (c) 2012 Computational Microbial Genomics Group, University of Glasgow, UK
% 
%            This program is free software: you can redistribute it and/or modify
%            it under the terms of the GNU General Public License as published by
%            the Free Software Foundation, either version 3 of the License, or
%            (at your option) any later version.
%
%            This program is distributed in the hope that it will be useful,
%            but WITHOUT ANY WARRANTY; without even the implied warranty of
%            MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%            GNU General Public License for more details.
%
%            You should have received a copy of the GNU General Public License
%            along with this program.  If not, see <http://www.gnu.org/licenses/>.
% **************************************************************/

logTransformData=1;%set to 0 if you dont want to transform the data

lineArray=read_mixed_csv(fileName,',');
[rows cols]=size(lineArray);
data=zeros(rows-1,cols-1);
for i=1:rows
    for j=1:cols
        if(i==1 && j>=2)
            sample_names{j-1}=lineArray{i,j};
        end
        if(j==1 && i>=2)
            OTU_names{i-1}=lineArray{i,j};
        end
        if ((i>=2) && (j>=2))
            data(i-1,j-1)=str2num(lineArray{i,j});
        end
    end
end
if(logTransformData)
    data=transpose(log(double(data)+1));
else
    data=transpose(data);
end
mu=mean(data,1);
X_minus_mu=data-repmat(mu, size(data,1), 1);
sigma=(X_minus_mu'*X_minus_mu)/size(data,1);
AM=L1precisionBCD(sigma,1);

%generate dot graph%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
adj=AM;
adj(abs(adj) < 1e-4) = 0;
adj = abs(sign(adj));

labels=transpose(OTU_names);

adj = setdiag(adj, 0); %remove self loops
[n,m] = size(adj);

if isempty(labels)
  labels = cell(1,n);
  for i=1:n
    labels{i} = sprintf('%d', i); 
  end
end

removeIsolatedNodes=1;

if removeIsolatedNodes
  isolated = [];
  for i=1:n
    nbrs = [find(adj(i,:)) find(adj(:,i))'];
    nbrs = setdiff(nbrs, i);
    if isempty(nbrs)
      isolated = [isolated i];
    end
  end
  adj = removeRowsCols(adj, isolated, isolated);
  labels(isolated) = [];
end

adj = double(adj > 0);    % make sure it is a binary matrix cast to double type

graph_to_dot(adj, 'directed', 0, ...
	     'filename', strcat(fileName(1:end-3),'dot'), 'node_label', labels);
%/generate dot graph%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     

function lineArray = read_mixed_csv(fileName,delimiter)
  fid = fopen(fileName,'r');   %# Open the file
  lineArray = cell(100,1);     %# Preallocate a cell array (ideally slightly
                               %#   larger than is needed)
  lineIndex = 1;               %# Index of cell to place the next line in
  nextLine = fgetl(fid);       %# Read the first line from the file
  while ~isequal(nextLine,-1)         %# Loop while not at the end of the file
    lineArray{lineIndex} = nextLine;  %# Add the line to the cell array
    lineIndex = lineIndex+1;          %# Increment the line index
    nextLine = fgetl(fid);            %# Read the next line from the file
  end
  fclose(fid);                 %# Close the file
  lineArray = lineArray(1:lineIndex-1);  %# Remove empty cells, if needed
  for iLine = 1:lineIndex-1              %# Loop over lines
    lineData = textscan(lineArray{iLine},'%s',...  %# Read strings
                        'Delimiter',delimiter);
    lineData = lineData{1};              %# Remove cell encapsulation
    if strcmp(lineArray{iLine}(end),delimiter)  %# Account for when the line
      lineData{end+1} = '';                     %#   ends with a delimiter
    end
    lineArray(iLine,1:numel(lineData)) = lineData;  %# Overwrite line data
  end

function [w,wp,m] = LassoShooting(X, y, lambda,varargin)
% This function computes the Least Squares parameters
% with a penalty on the L1-norm of the parameters
%
% Method used:
%   The Shooting method of [Fu, 1998]
%
% Modifications:
%   We precompute the Hessian diagonals, since they do not 
%   change between iterations
[maxIter,verbose,optTol,zeroThreshold] = process_options(varargin,'maxIter',10000,'verbose',2,'optTol',1e-5,'zeroThreshold',1e-4);
[n p] = size(X);

% Start from the Least Squares solution
beta = (X'*X + lambda*eye(p))\(X'*y);
% Start the log
if verbose==2
    w_old = beta;
    fprintf('%10s %10s %15s %15s %15s\n','iter','shoots','n(w)','n(step)','f(w)');
    k=1;
    wp = beta;
end

m = 0;

XX2 = X'*X*2;
Xy2 = X'*y*2;
while m < maxIter
    
    
    
    beta_old = beta;
    for j = 1:p
        
        % Compute the Shoot and Update the variable
        S0 = sum(XX2(j,:)*beta) - XX2(j,j)*beta(j) - Xy2(j);
        if S0 > lambda
            beta(j,1) = (lambda - S0)/XX2(j,j);
        elseif S0 < -lambda
            beta(j,1) = (-lambda - S0)/XX2(j,j);
        elseif abs(S0) <= lambda
            beta(j,1) = 0;
        end
        
    end
    
    m = m + 1;
    
    % Update the log
    if verbose==2
        fprintf('%10d %10d %15.2e %15.2e %15.2e\n',m,m*p,sum(abs(beta)),sum(abs(beta-w_old)),...
            sum((X*beta-y).^2)+lambda*sum(abs(beta)));
        w_old = beta;
        k=k+1;
        wp(:,k) = beta;
    end
    % Check termination
    if sum(abs(beta-beta_old)) < optTol
        break;
    end
    
    
end
if verbose
fprintf('Number of iterations: %d\nTotal Shoots: %d\n',m,m*p);
end
w = beta;

function [X] = L1precisionBCD(sigma_emp,lambda)

if nargin < 3
    useQP = 1;
end

verbose = 1;
optTol = 0.00001;
S = sigma_emp;
p = size(S,1);
row = lambda;
maxIter = 10;
A = [eye(p-1,p-1);-eye(p-1,p-1)];
f = zeros(p-1,1);

% Initial W
W = S + row*eye(p,p);

% Check for qp mex file
if exist('qpas') == 3
    useQP = 1;
    qpSolver = @qpas;
    qpArgs = {[],[],[],[]};
elseif 1
    useQP = 0;
else
    useQP = 1;
    qpSolver = @quadprog;
    options = optimset('LargeScale','off','Display','none');
    qpArgs = {[],[],[],[],[],options};
end

for iter = 1:maxIter

    % Check Primal-Dual gap
    X = W^-1; % W should be PD
    gap = trace(S*X) + row*sum(sum(abs(X))) - p;
    fprintf('Iter = %d, OptCond = %.5f\n',iter,gap);
    if gap < optTol
        fprintf('Solution Found\n');
        break;
    end

    for i = 1:p

        if verbose
            X = W^-1; % W should be PD
            gap = trace(S*X) + row*sum(sum(abs(X))) - p;
            fprintf('Column = %d, OptCond = %.5f\n',i,gap);
            if gap < optTol
                fprintf('Solution Found\n');
                break;
            end
        end

        % Compute Needed Partitions of W and S
        s_12 = S(mysetdiff(1:p,i),i);
        
        if useQP
            % Solve as QP
            H = 2*W(mysetdiff(1:p,i),mysetdiff(1:p,i))^-1;
            b = row*ones(2*(p-1),1) + [s_12;-s_12];
            w = qpSolver((H+H')/2,f,A,b,qpArgs{:});
        else
            % Solve with Shooting
            W_11 = W(mysetdiff(1:p,i),mysetdiff(1:p,i));
            Xsub = sqrtm(W_11);
            ysub = Xsub\s_12;
            w = W_11*LassoShooting(Xsub,ysub,2*row,'verbose',0);
        end

        % Un-Permute
        W(mysetdiff(1:p,i),i) = w;
        W(i,mysetdiff(1:p,i)) = w';
    end
    %drawnow
end

function graph_to_dot(adj, varargin)

% graph_to_dot(adj, VARARGIN)  Creates a GraphViz (AT&T) format file representing 
%                     a graph given by an adjacency matrix.
%  Optional arguments should be passed as name/value pairs [default]
%
%   'filename'  -  if omitted, writes to 'tmp.dot'
%  'arc_label'  -  arc_label{i,j} is a string attached to the i-j arc [""]
% 'node_label'  -  node_label{i} is a string attached to the node i ["i"]
%  'width'      -  width in inches [10]
%  'height'     -  height in inches [10]
%  'leftright'  -  1 means layout left-to-right, 0 means top-to-bottom [0]
%  'directed'   -  1 means use directed arcs, 0 means undirected [1]
%
% For details on dotty, See http://www.research.att.com/sw/tools/graphviz
%
% by Dr. Leon Peshkin, Jan 2004      inspired by Kevin Murphy's  BNT
%    pesha @ ai.mit.edu /~pesha
                   
node_label = [];   arc_label = [];   % set default args
width = 10;        height = 10;
leftright = 0;     directed = 1;     filename = 'tmp.dot';
           
for i = 1:2:nargin-1              % get optional args
    switch varargin{i}
        case 'filename', filename = varargin{i+1};
        case 'node_label', node_label = varargin{i+1};
        case 'arc_label', arc_label = varargin{i+1};
        case 'width', width = varargin{i+1};
        case 'height', height = varargin{i+1};
        case 'leftright', leftright = varargin{i+1};
        case 'directed', directed = varargin{i+1};
    end
end
fid = fopen(filename, 'w');
if fid==-1
  error(sprintf('could not write to %s', filename))
end
if directed
    fprintf(fid, 'digraph G {\n');
    arctxt = '->'; 
    if isempty(arc_label)
        labeltxt = '';
    else
        labeltxt = '[label="%s"]';
    end
else
    fprintf(fid, 'graph G {\n');
    arctxt = '--'; 
    if isempty(arc_label)
        labeltxt = '[dir=none]';
    else
        labeltext = '[label="%s",dir=none]';
    end
end
fprintf(fid, 'center = 1;\n');
fprintf(fid, 'size=\"%d,%d\";\n', width, height);
if leftright
    fprintf(fid, 'rankdir=LR;\n');
end
Nnds = length(adj);
for node = 1:Nnds               % process NODEs 
    if isempty(node_label)
        fprintf(fid, '%d;\n', node);
    else
        fprintf(fid, '%d [ label = "%s" ];\n', node, node_label{node});
    end
end
edgeformat = strcat(['%d ',arctxt,' %d ',labeltxt,';\n']);
for node1 = 1:Nnds              % process ARCs
    if directed
        arcs = find(adj(node1,:));         % children(adj, node);
    else
        arcs = find(adj(node1,node1+1:Nnds)) + node1; % remove duplicate arcs
    end
    for node2 = arcs
        fprintf(fid, edgeformat, node1, node2);
    end
end
fprintf(fid, '}'); 
fclose(fid);

function M = removeRowsCols(M, rows, cols)
% Remove rows and columns from a matrix
% Example
% M = reshape(1:25,[5 5])
%> removeRowsCols(M, [2 3], 4)
%ans =
%     1     6    11    21
%     4     9    14    24
%     5    10    15    25
     
[nr nc] = size(M);

ndx = [];
for i=1:length(rows)
  tmp = repmat(rows(i), nc, 1);
  tmp2 = [tmp (1:nc)'];
  ndx = [ndx; tmp2];
end
for i=1:length(cols)
  tmp = repmat(cols(i), nr, 1);
  tmp2 = [(1:nr)' tmp];
  ndx = [ndx; tmp2];
end
if isempty(ndx), return; end
k = subv2ind([nr nc], ndx);
M(k) = [];
M = reshape(M, [nr-length(rows) nc-length(cols)]);

%%%%%%%%%

function ndx = subv2ind(siz, subv)
% SUBV2IND Like the built-in sub2ind, but the subscripts are given as row vectors.
% ind = subv2ind(siz,subv)
%
% siz can be a row or column vector of size d.
% subv should be a collection of N row vectors of size d.
% ind will be of size N * 1.
%
% Example:
% subv = [1 1 1;
%         2 1 1;
%         ...
%         2 2 2];
% subv2ind([2 2 2], subv) returns [1 2 ... 8]'
% i.e., the leftmost digit toggles fastest.
%
% See also IND2SUBV.

 
if isempty(subv)
  ndx = [];
  return;
end

if isempty(siz)
  ndx = 1;
  return;
end

[ncases ndims] = size(subv);

if all(siz==2)
  twos = pow2(0:ndims-1);
  ndx = ((subv-1) * twos(:)) + 1;
else
  cp = [1 cumprod(siz(1:end-1))]';
  ndx = (subv-1)*cp + 1;
end

%%%%%%%%%%%

function M = setdiag(M, v)
% SETDIAG Set the diagonal of a matrix to a specified scalar/vector.
% M = set_diag(M, v)

n = length(M);
if length(v)==1
  v = repmat(v, 1, n);
end

% e.g., for 3x3 matrix,  elements are numbered
% 1 4 7 
% 2 5 8 
% 3 6 9
% so diagnoal = [1 5 9]


J = 1:n+1:n^2;
M(J) = v;

function C = mysetdiff(A,B)
% MYSETDIFF Set difference of two sets of positive integers (much faster than built-in setdiff)
% C = mysetdiff(A,B)
% C = A \ B = { things in A that are not in B }
%
% Original by Kevin Murphy, modified by Leon Peshkin

if isempty(A)
    C = [];
    return;
elseif isempty(B)
    C = A;
    return; 
else % both non-empty
    bits = zeros(1, max(max(A), max(B)));
    bits(A) = 1;
    bits(B) = 0;
    C = A(logical(bits(A)));
end

function [varargout] = process_options(args, varargin)

% PROCESS_OPTIONS - Processes options passed to a Matlab function.
%                   This function provides a simple means of
%                   parsing attribute-value options.  Each option is
%                   named by a unique string and is given a default
%                   value.
%
% Usage:  [var1, var2, ..., varn[, unused]] = ...
%           process_options(args, ...
%                           str1, def1, str2, def2, ..., strn, defn)
%
% Arguments:   
%            args            - a cell array of input arguments, such
%                              as that provided by VARARGIN.  Its contents
%                              should alternate between strings and
%                              values.
%            str1, ..., strn - Strings that are associated with a 
%                              particular variable
%            def1, ..., defn - Default values returned if no option
%                              is supplied
%
% Returns:
%            var1, ..., varn - values to be assigned to variables
%            unused          - an optional cell array of those 
%                              string-value pairs that were unused;
%                              if this is not supplied, then a
%                              warning will be issued for each
%                              option in args that lacked a match.
%
% Examples:
%
% Suppose we wish to define a Matlab function 'func' that has
% required parameters x and y, and optional arguments 'u' and 'v'.
% With the definition
%
%   function y = func(x, y, varargin)
%
%     [u, v] = process_options(varargin, 'u', 0, 'v', 1);
%
% calling func(0, 1, 'v', 2) will assign 0 to x, 1 to y, 0 to u, and 2
% to v.  The parameter names are insensitive to case; calling 
% func(0, 1, 'V', 2) has the same effect.  The function call
% 
%   func(0, 1, 'u', 5, 'z', 2);
%
% will result in u having the value 5 and v having value 1, but
% will issue a warning that the 'z' option has not been used.  On
% the other hand, if func is defined as
%
%   function y = func(x, y, varargin)
%
%     [u, v, unused_args] = process_options(varargin, 'u', 0, 'v', 1);
%
% then the call func(0, 1, 'u', 5, 'z', 2) will yield no warning,
% and unused_args will have the value {'z', 2}.  This behaviour is
% useful for functions with options that invoke other functions
% with options; all options can be passed to the outer function and
% its unprocessed arguments can be passed to the inner function.

% Copyright (C) 2002 Mark A. Paskin
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
% USA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Check the number of input arguments
n = length(varargin);
if (mod(n, 2))
  error('Each option must be a string/value pair.');
end

% Check the number of supplied output arguments
if (nargout < (n / 2))
  error('Insufficient number of output arguments given');
elseif (nargout == (n / 2))
  warn = 1;
  nout = n / 2;
else
  warn = 0;
  nout = n / 2 + 1;
end

% Set outputs to be defaults
varargout = cell(1, nout);
for i=2:2:n
  varargout{i/2} = varargin{i};
end

% Now process all arguments
nunused = 0;
for i=1:2:length(args)
  found = 0;
  for j=1:2:n
    if strcmpi(args{i}, varargin{j})
      varargout{(j + 1)/2} = args{i + 1};
      found = 1;
      break;
    end
  end
  if (~found)
    if (warn)
      warning(sprintf('Option ''%s'' not used.', args{i}));
      args{i}
    else
      nunused = nunused + 1;
      unused{2 * nunused - 1} = args{i};
      unused{2 * nunused} = args{i + 1};
    end
  end
end

% Assign the unused arguments
if (~warn)
  if (nunused)
    varargout{nout} = unused;
  else
    varargout{nout} = cell(0);
  end
end

